/*
 * afflib_dir.cpp:
 * 
 * Functions for the manipulation of the AFF directories
 */

#include "afflib.h"
#include "afflib_i.h"

int	af_dir_free(AFFILE *af)
{
    if(af->dir){
	for(int i=0;i<af->dir_count;i++){
	    if(af->dir[i].name) free(af->dir[i].name);
	}
	free(af->dir);
	af->dir = 0;
	af->dir_count = 0;
    }
    return 0;
}


void	af_dir_print(AFFILE *af)
{
    printf("AF DIRECTORY:\n");
    for(int i=0;i<af->dir_count;i++){
	if(af->dir[i].name){
	    printf("%-32s %" I64d " \n",af->dir[i].name, af->dir[i].offset);
	}
    }
}

void	af_dir_append(AFFILE *af,const char *segname,int64 offset)
{
    af->dir = (af_dir_mem *)realloc(af->dir,sizeof(*af->dir)*(af->dir_count+1));
    af->dir[af->dir_count].offset = offset;

    af->dir[af->dir_count].name = strdup(segname); // make a copy of the string
    af->dir_count++;
}

/*
 * af_dir_build:
 * Build the directory by reading the existing file...
 */
int	af_dir_build(AFFILE *af)	// build the dir if we couldn't find it
{
    return 0;				// don't read existing dirs for now
    af_dir_free(af);
    af_rewindi(af);			// start at the beginning
    af->dir = (af_dir_mem *)malloc(0);
    while(!af_seg_eof(af)){
	char segname[AF_MAX_NAME_LEN];
	size_t segname_len=sizeof(segname);
	int64 pos = ftello(af->aseg);

	if(af_get_next_seg(af,segname,segname_len,0,0,0)!=0) break;
	af_dir_append(af,segname,pos);
    }
    return 0;
}

int	af_dir_read(AFFILE *af)	// find the directory in the file and read it
{
    return 0;
}

/*
 * The easiest way to write the directory is to create it in memory first and then
 * write it out as a signle operation.
 */
int	af_dir_write(AFFILE *af)	// write out the dir
{
    af_del_seg(af,AF_DIRECTORY);	// delete the directory segment

    /* This is a two-pass algorithm. First pass through we calculate the size. 
     * Second pass through we write it.
     */
    size_t needed = 0;
    for(int i=0;i<af->dir_count;i++){
	needed += 4 + 8 + strlen(af->dir[i].name);
    }

    unsigned char *buf = (unsigned char *)malloc(needed);
    if(buf==0) return 0;		// memory allocation failed
    unsigned char *b = buf;
    for(int i=0;i<af->dir_count;i++){
	if(af->dir[i].name){
	    size_t name_len = strlen(af->dir[i].name);
	    *(unsigned long *)b = htonl(4 + 8 + name_len);
	    b += 4;				// space for the length of entry
	    ((struct aff_quad *)b)->low  = htonl((unsigned long)(af->dir[i].offset & 0xffffffff));
	    ((struct aff_quad *)b)->high = htonl((unsigned long)(af->dir[i].offset >> 32));
	    b += 8;				// space for the quad offset
	    memcpy(b,af->dir[i].name,name_len); // copy the name
	    b += name_len;
	}
    }

    fseeko(af->aseg,0L,SEEK_END);	// directory should be last segment
    af_write_seg(af,AF_DIRECTORY,0,buf,needed); // write out the directory
    free(buf);				// free the memory copy
    return 0;				// always succedes
}


/*
 * return the named entry in the directory
 */

struct af_dir_mem *af_dir(AFFILE *af,const char *segname)
{
    if(af->dir){
	for(int i=0;i<af->dir_count;i++){
	    if(af->dir[i].name && strcmp(af->dir[i].name,segname)==0) return &af->dir[i];
	}
    }
    return 0;
}

/* Delete something from the directory, but don't bother reallocating.*/
void af_dir_del(AFFILE *af,const char *segname)
{
    if(af->dir){
	for(int i=0;i<af->dir_count;i++){
	    if(strcmp(af->dir[i].name,segname)==0){
		free(af->dir[i].name);
		af->dir[i].name=0;
		return;
	    }
	}
    }
}

void af_dir_insert(AFFILE *af,const char *segname,int64 offset)
{
    if(af->dir){
	for(int i=0;i<af->dir_count;i++){
	    if(af->dir[i].name==0 || strcmp(af->dir[i].name,segname)==0){
		if(af->dir[i].name==0){	// if name was empty, copy it over
		    af->dir[i].name = strdup(segname);
		}
		af->dir[i].offset = offset;
		return;
	    }
	}
	/* Need to append it to the directory */
	af_dir_append(af,segname,offset);
    }
}
