#include "afflib.h"
#include "timer.h"
#include <openssl/md5.h>
#include <openssl/sha.h>


class imager {
public:
    bool  allow_regular;		// allow the imaging of a regular file
    uint64 total_sectors_written ;
    uint64 total_sectors_read;
    uint64 total_bytes_read;
    uint64 total_bytes_written;
    uint64 total_blank_sectors;

    bool	imaging;

    /* Timers */
    timer	reading_timer;
    timer	writing_timer;


    /* Device parameters */
    int		in;			// input fd
    int		sector_size;		// in bytes; 0 if unknown
    uint64	total_sectors;	      // in sectors; 0 if uncomputable
    int		maxreadblocks;		// in bytes

    AFFILE *af;				// AF output file
    FILE *fout;				// STDIO output file

    /* For calculating hashes */
    MD5_CTX md5;			// for the image
    SHA_CTX sha;			// for the image
    bool hash_invalid;			// did we reverse direction or skip?


    uint64	last_sector_read ;	// sector number last read
    int		last_sectors_read;

    /* Configuration */
    bool seek_on_output;		// if True, then we need to recalculate md5 & sha
    int  retry_count;

    imager();
    void (*status)();
    void image_loop(uint64 low_water_mark,
			uint64 high_water_mark,
			int direction, int readsectors,int error_mask);
    int set_input(int ifd);
    void set_fout(FILE *fout);
    void write_data(unsigned char *buf,uint64 offset,int bytes_read);
    void set_af(AFFILE *af);
    void hash_setup();
    float compression_ratio();

    char  blank_sector[512];		// a lot of zeros
    int   partial_sector_left;		//
    bool  partial_sector_blank;

    /* Error handling */
    uint64 bad_sectors_read;
    int errors_in_a_row;
    int error_recovery_phase;
    int last_direction;			// 1 = forwards, -1 = backwards
};


